﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace WRG315Example
{
    public partial class MainForm : Form
    {
        enum DeviceStatus { Disconnected, Connected, Unknown};

        private DeviceStatus deviceStatus = DeviceStatus.Unknown;

        #region Constructors

        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Private Methods

        private void InitializeDeviceConnection()
        {
            // Get Radio Info
            Wrapper.RadioInfo anInfo = new Wrapper.RadioInfo();
            int aStructSize = Marshal.SizeOf(anInfo), anInfoSize = 0;
            anInfo.length = (uint)aStructSize;

            if (Wrapper.GetRadioList(ref anInfo, aStructSize, ref anInfoSize) == 1)
            {
                textBoxProductName.Text = anInfo.productName;
                textBoxSerialNumber.Text = anInfo.serialNumber;

                trackBarFrequency.Minimum = (int)anInfo.minFrequency;
                trackBarFrequency.Maximum = (int)anInfo.maxFrequency;
                textBoxFrequencyRange.Text = trackBarFrequency.Minimum + " to " + trackBarFrequency.Maximum;
                textBoxFrequency.Text = trackBarFrequency.Value.ToString();
            }

            // Open device
            myRadioHandle = Wrapper.OpenRadioDevice(0);

            CheckDeviceConnection();
        }

        private void CheckDeviceConnection()
        {
            bool anIsDeviceConnected = Wrapper.IsDeviceConnected(myRadioHandle);

            if (deviceStatus == DeviceStatus.Unknown ||
                deviceStatus == DeviceStatus.Disconnected && anIsDeviceConnected ||
                deviceStatus == DeviceStatus.Connected && !anIsDeviceConnected)
            {
                if (anIsDeviceConnected)
                {
                    toolStripStatusLabel.ForeColor = Color.Black;
                    toolStripStatusLabel.Text = "Device #" + myRadioHandle + " opened";

                    buttonConnection.Visible = false;

                    deviceStatus = DeviceStatus.Connected;

                    timerConnection.Start();
                }
                else
                {
                    timerConnection.Stop();

                    Wrapper.CloseRadioDevice(myRadioHandle);

                    toolStripStatusLabel.ForeColor = Color.Red;
                    toolStripStatusLabel.Text = "Device not opened";

                    textBoxProductName.Text = "";
                    textBoxSerialNumber.Text = "";
                    textBoxFrequencyRange.Text = "";
                    trackBarFrequency.Value = trackBarFrequency.Minimum;
                    textBoxFrequency.Text = "";

                    buttonConnection.Visible = true;

                    deviceStatus = DeviceStatus.Disconnected;
                }

                panelMain.Enabled = trackBarFrequency.Enabled = anIsDeviceConnected;
            }
        }

        #endregion

        #region Event Handlers

        private void MainFormLoad(object theSender, EventArgs theEventArgs)
        {
            InitializeDeviceConnection();
        }

        private void MainFormClosing(object theSender, FormClosingEventArgs theEventArgs)
        {
            timerConnection.Stop();

            Wrapper.CloseRadioDevice(myRadioHandle);

            if (!Wrapper.IsDeviceConnected(myRadioHandle))
            {
                toolStripStatusLabel.Text = "Device closed...";
            }
        }

        private void TrackBarFrequencyValueChanged(object theSender, EventArgs theEventArgs)
        {
            if (deviceStatus == DeviceStatus.Connected)
            {
                Wrapper.SetFrequency(myRadioHandle, trackBarFrequency.Value);
                textBoxFrequency.Text = Wrapper.GetFrequency(myRadioHandle).ToString();
            }
        }

        void TimerConnectionTick(object theSender, EventArgs theEventArgs)
        {
            CheckDeviceConnection();
        }

        private void ButtonConnectionClick(object theSender, EventArgs theEventArgs)
        {
            InitializeDeviceConnection();
        }

        #endregion
    }
}